<?php

require_once("MyCSV.class.php");
require_once("Apeform.class.php");

/*
 * LICENSE
 * 1. If you like to use this class for personal purposes, it's free.
 * 2. For comercial purposes, please contact me (http://email.maettig.com/).
 *    I'll send a license to you.
 * 3. When you copy the framework you must copy this notice with the source
 *    code. You may alter the source code, but you have to put the original
 *    with your altered version.
 * 4. The license is for all files included in this bundle.
 *
 * TM::Tracker Change Log
 *
 * 2003-11-11
 *  * The "add comment" form doesn't grab the focus any more.
 * 2003-11-10
 *  * Version and description aren't required. Not displayed when empty.
 *  * Typos fixed.
 * 2003-11-03
 *  * "Reported" and "updated" date added.
 *  * Colors added. Detail view uses CSS'ed <dl> now.
 *
 * KNOWN BUGS/TODO
 * - ...
 */

/**
 * Bug and feature request tool. Demo application for the MyCSV and Apeform
 * classes.
 *
 * @author Thiemo Mttig (http://maettig.com/)
 * @version 2003-10-30
 * @package TM
 */
class Tracker
{
    var $html = "";
    var $bugs = array();
    var $categories = array();
    var $status = array();

    /**
     * Initializes the tracker.
     *
     * @return Tracker
     */
    function Tracker()
    {
        global $_REQUEST;

        $this->bugs = new MyCSV("admin/tracker");
        $this->categories = new MyCSV("admin/category");
        $this->categories->sort("category SORT_STRING");
        $this->status = new MyCSV("admin/status");

        $method = isset($_REQUEST['method']) ? $_REQUEST['method'] : "";
        if (! method_exists(&$this, $method)) $method = "browse";
        call_user_func(array(&$this, $method));
    }

    /**
     * @return void
     */
    function browse()
    {
        global $_REQUEST;

        $this->html .= '<p><a href="{SELF}?method=add">';
        $this->html .= '&gt;&gt; Report new Bug/Feature Request</a></p>';

        $this->bugs->join(&$this->categories, "category_id");
        $this->bugs->join(&$this->status, "status_id");
        $this->html .= '<table><tr>';
        $ths = array(
            "id" => "ID",
            "" => "Summary",
            "category_id" => "Category",
            "version SORT_NAT" => "Version",
            "updated" => "Updated",
            "status_id" => "Status"
        );
        foreach ($ths as $sort => $th)
        {
            $this->html .= '<th>';
            if (! $sort) $this->html .= $th;
            else
            {
                if (isset($_REQUEST['sort']) && $sort == $_REQUEST['sort']) $sort .= " DESC";
                $this->html .= '<a href="{SELF}?sort=' . urlencode($sort) . '">';
                $this->html .= $th . '</a>';
            }
            $this->html .= '</th>';
        }
        $this->html .= '</tr>';
        if (! empty($_REQUEST['sort'])) $this->bugs->sort($_REQUEST['sort']);
        while ($row = $this->bugs->each())
        {
            $class = empty($row['style']) ? '' : ' class="status' . $row['status_id'] . '"';
            $aHref = '<a href="{SELF}?method=detail&id=' . $row['id'] . '">';
            $this->html .= '<tr>';
            $this->html .= '<td' . $class . '>' . $aHref . '#' . $row['id'] . '</a></td>';
            $this->html .= '<td' . $class . '>' . $aHref . htmlspecialchars($row['summary']) . '</a></td>';
            $this->html .= '<td' . $class . '>' . $row['category'] . '</td>';
            $this->html .= '<td' . $class . '>' . htmlspecialchars($row['version']) . '</td>';
            $this->html .= '<td' . $class . '>' . $row['updated'] . '</td>';
            $this->html .= '<td' . $class . '>' . $row['status'] . '</td>';
            $this->html .= '</tr>';
        }
        $this->html .= '</table>';
    }

    /**
     * @return void
     */
    function detail()
    {
        global $_REQUEST;

        if (! $bug = $this->bugs->data($_REQUEST['id']))
        {
            $this->browse();
            return;
        }

        $form = new Apeform(0, 60, false);
        $form->hidden("detail", "method");
        $form->hidden($bug['id'], "id");
        $data['email'] = &$form->text("Your <u>N</u>ame/Email", "Your Name and/or Email Address (won't be displayed)");
        if (! $data['email']) $form->error("Please enter at least your Name");
        if (strlen($data['email']) > 255) $form->error("Email/Name to long");
        $data['comment'] = &$form->textarea("<u>C</u>omment", "", "", 10);
        $data['comment'] = trim(preg_replace('/[ \t]*(\r*\n|\r)/s', "\n", $data['comment']));
        if (! $data['comment']) $form->error("Please enter your Comment");
        if (strlen($data['comment']) > 5120) $form->error("Comment to long (max. 5KB)");
        $form->submit("Add Comment");

        $comments = new MyCSV("admin/comments");

        $data['tracker_id'] = $bug['id'];
        if ($form->isValid() && ! $comments->row_exists($data))
        {
            $data['date'] = date("Y-m-d H:i");
            $comments->insert($data);
            $comments->write();

            $bug['updated'] = $data['date'];
            $this->bugs->update($bug);
            $this->bugs->write();
        }

        $this->html .= '<h2>Bug/Feature Request #' . $bug['id'];
        if ($bug['summary']) $this->html .= ': ' . htmlspecialchars($bug['summary']);
        $this->html .= '</h2>';

        $this->html .= '<dl>';
        $this->html .= '<dt>Category</dt>';
        $this->html .= '<dd>' . $this->categories->data[$bug['category_id']]['category'];
        if ($bug['version'])
            $this->html .= ', Version ' . htmlspecialchars($bug['version']);
        $this->html .= '</dd>';
        $this->html .= '<dt>Reported</dt>';
        $this->html .= '<dd>' . $bug['date'];
        if ($bug['date'] != $bug['updated']) $this->html .= ', updated ' . $bug['updated'];
        $this->html .= '</dd>';
        $this->html .= '<dt>Status</dt>';
        $this->html .= '<dd class="status' . $bug['status_id'] . '">';
        $this->html .= $this->status->data[$bug['status_id']]['status'] . '</dd>';
        $this->html .= '</dl>';

        $comments->reset();
        while ($row = $comments->each())
        {
            if ($row['tracker_id'] != $bug['id']) continue;

            $row['email'] = preg_replace('/@.*?(\.([a-z]{2,4}))?$/i', ' at ...\2', $row['email']);
            $row['email'] = htmlspecialchars(ucfirst($row['email']));
            $this->html .= '<h3>[' . $row['date'] . '] ' . $row['email'] . '</h3>';

            $row['comment'] = htmlspecialchars(trim($row['comment']));
            $row['comment'] = str_replace("@", "&#x40;", $row['comment']);
            $row['comment'] = preg_replace('/\n{3,}/s', "\n\n", $row['comment']);
            $row['comment'] = preg_replace('/(\s) /s', '\1&nbsp;', $row['comment']);
            $row['comment'] = preg_replace(
                '{\b(https?://|(www\.))([^\s"\'<>()\[\]\\\]{4,}[\w/=])}is',
                '<a href="http://\2\3" target="_blank">\0</a>',
            $row['comment']);
            $row['comment'] = preg_replace('/(?<!\w)#(\d+)\b/', '<a href="{SELF}?method=detail&id=\1">\0</a>', $row['comment']);
            $row['comment'] = nl2br($row['comment']);
            $this->html .= '<p class="comment">' . $row['comment'] . '</p>';
        }

        if (! $form->isValid())
        {
            $this->html .= '<h3>Add new Comment</h3>';
            $this->html .= $form->toHtml(false);
        }

        $this->html .= '<p><a href="{SELF}">&lt;&lt; Back</a></p>';
    }

    /**
     * @return void
     */
    function add()
    {
        $form = new Apeform(0, 60, false);
        $form->hidden("add", "method");
        $options = array("Select Category");
        while ($row = $this->categories->each())
            $options[$row['id']] = $row['category'];
        $data['category_id'] = $form->select("<u>C</u>ategory", "", $options);
        if (! $data['category_id']) $form->error("Please select a Category");
        if (! isset($this->categories->data[$data['category_id']]))
            $form->error("Category doesn't exist");
        $data['version'] = &$form->text("<u>V</u>ersion", 'Version Number or Release Date, e.g. "2003-12-31"');
        if (strlen($data['version']) > 255) $form->error("Version/Date to long");
        $email = &$form->text("Your <u>N</u>ame/Email", "Your Name and/or Email Address (won't be displayed)");
        if (! $email) $form->error("Please enter at least your Name");
        if (strlen($email) > 255) $form->error("Email/Name to long");
        $data['summary'] = &$form->text("<u>S</u>ummary");
        if (! $data['summary']) $form->error("Please enter a Summary");
        if (strlen($data['summary']) > 255) $form->error("Summary to long");
        $comment = &$form->textarea("<u>D</u>escription", "Please add detailed Description, reproduce Code, expected and actual Results, ...", "", 20);
        $comment = trim(preg_replace('/[ \t]*(\r*\n|\r)/s', "\n", $comment));
        if (strlen($comment) > 5120) $form->error("Description to long (max. 5KB)");
        $form->submit("Report Bug/Feature Request");

        if (! $form->isValid())
        {
            $this->html .= '<h2>Report new Bug/Feature request</h2>';
            $this->html .= $form->toHtml();
        }
        else
        {
            if (! $this->bugs->row_exists($data))
            {
                $data['status_id'] = 1;
                $data['date'] = date("Y-m-d H:i");
                $data['updated'] = $data['date'];
                $this->bugs->insert($data);
                $this->bugs->write();

                $comments = new MyCSV("admin/comments");
                $data['tracker_id'] = $this->bugs->insert_id();
                $data['email'] = $email;
                $data['comment'] = $comment;
                $comments->insert($data);
                $comments->write();
            }
            $this->browse();
        }

        $this->html .= '<p><a href="{SELF}">&lt;&lt; Back</a></p>';
    }

    /**
     * @return string
     */
    function toHtml()
    {
        $html = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>TM::Tracker</title>
<style type="text/css">
body{background-color:white;}
a{color:#006600;}
h1{font-size:150%}
h2,h3,th{background-color:#D2ECD2;}
td{background-color:#F4FAF4;}
small{color:#666666;}
dt{clear:both;float:left;font-weight:bold;text-align:right;width:10ex;}
dt:after{content:\':\';}
dd{margin-left:11ex;}
.error{color:#CC0000;font-weight:bold;}
.comment{font-family:monospace;max-width:75ex;}';
        while ($row = $this->status->each())
        {
            $html .= '.status' . $row['id'] . '{' . $row['style'] . '}';
        }
        $html .= '</style></head><body>';
        $html .= '<h1>TM::<a href="{SELF}">Tracker</a></h1>';
        $html .= $this->html;
        if (isset($_SERVER['PHP_SELF'])) $PHP_SELF = $_SERVER['PHP_SELF'];
        else $PHP_SELF = $GLOBALS['PHP_SELF'];
        $PHP_SELF = str_replace('index.php', '', $PHP_SELF);
        $html = preg_replace('/\{(PHP)?_?SELF\}/i', $PHP_SELF, $html);
        return $html;
    }

    /**
     * @return void
     */
    function display()
    {
        echo $this->toHtml();
    }
}

if (! isset($_REQUEST) && isset($HTTP_GET_VARS)) $_REQUEST = $HTTP_GET_VARS;

$tracker = new Tracker();
$tracker->display();

?>